#!/bin/bash
# This script will configure an existing VALT server to function as a 
# standby server.
# You will need to run VALTPrimaryServerConfig.sh on the primary server.

#primaryip="10.111.3.73" 
#logfile=""
#logpos=""

#echo "WARNING: DO NOT RUN THIS SCRIPT MULTIPLE TIMES!!!"
#echo "MAKE SURE YOU HAVE UPDATED THE LOGFILE, LOGPOS, AND primaryserveripaddress VARIABLES IN THIS SCRIPT!!"
#echo "MAKE SURE YOU RAN sudo -i PRIOR TO EXECUTING THIS SCRIPT!!"

# Load VALT Environment Variables
function LoadEnvironmentVariables() {
	source /usr/local/valt/conf/valtpath
	source $valtpath/.env.local
}

# Update VALT Environment Variables
function UpdateEnvLocal () {
	name=$1
	value="${2//\//\\/}"
	if [[ ! -f $valtpath/.env.local ]]; then
		echo -e "$1=$2" > $valtpath/.env.local
	else
		sed -i -e '/^#\?\(\s*'"${name}"'\s*=\s*\).*/{s//\1'"${value}"'/;:a;n;ba;q}' -e '$a'"${name}"'='"${value}" $valtpath/.env.local
	fi
}

# Update Environment Variables with Warm Standby variables
function UpdateWarmStandbyVariables() {
	UpdateEnvLocal "IS_PRIMARY_SERVER" "$isPrimaryServer"
	UpdateEnvLocal "IS_STANDBY_SERVER" "$isStandbyServer"
	UpdateEnvLocal "PRIMARY_SERVER_IP_ADDRESS" "$primaryserveripaddress"
	UpdateEnvLocal "STANDBY_SERVER_IP_ADDRESS" "$standbyserveripaddress"
	UpdateEnvLocal "SLAVE_USER" "$slaveuser"
	UpdateEnvLocal "SLAVE_PASSWORD" "$slavepassword"
}

function ConfigureStandbyServer() {
	echo "Please run VALTPrimaryServerConfig5.5.sh on the desired Primary Standby Server before executing this script."
	echo "Press CTRL-C to cancel or press enter to continue."
	read r

	export NCURSES_NO_UTF8_ACS=1

	if [[ $EUID -ne 0 ]]; then
		echo "You must run this script as root" 2>&1
		exit 1
	else
		# Verify we have a Primary Standby Server Config file to take data from
		if [[ -e /tmp/primaryserver ]]; then			
			# Get Primary Standby Server Config variables
			source /tmp/primaryserver
				# $masterlogfile
				# $masterlogposition
				# $primaryserveripaddress
				# $standbyserveripaddress
				# $sslenabled
				
			# Initialize condition variables
			isPrimaryServer=false
			isStandbyServer=false
			primaryuser="ivsadmin" 
			primarypass="admin51" 
			slaveuser="ivs_user"
			slavepassword="admin51"

			# Update .env.local with initialized Warm Standby Config
			UpdateWarmStandbyVariables		
			
			# Load VALT Environment variables after updating them
			LoadEnvironmentVariables
			
			# Configure Standby Server
			apt install rsync
			
			ssh-keygen -t rsa -f /root/.ssh/id_rsa -q -N ""
			ssh-copy-id -i /root/.ssh/id_rsa.pub ivsadmin@$PRIMARY_SERVER_IP_ADDRESS
			
			rsync --rsync-path="sudo rsync" -avz --progress ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/WowzaStreamingEngine/content/valt_recordings /usr/local/WowzaStreamingEngine/content/
			(crontab -l ; echo -e '00 11 * * * rsync --rsync-path="sudo rsync" -avz ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/WowzaStreamingEngine/content/valt_recordings /usr/local/WowzaStreamingEngine/content/') | crontab -
			service apache2 stop
			service WowzaStreamingEngine stop
			service valtcontrols stop
			systemctl disable apache2 
			systemctl disable valtcontrols
			systemctl disable WowzaStreamingEngine
			scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/WowzaStreamingEngine/conf/dustin/Application.xml /usr/local/WowzaStreamingEngine/conf/dustin/Application.xml
			
			#*********************************************************************
			# The section below is for HTTPS enabled environments only
			#*********************************************************************
			if [[ $sslenabled == "true" ]]; then
				# Move ssl config file to VALt Config directory
				mv /tmp/ssl /usr/local/valt/conf
				
				#Deteremine TYPE of SSL
				# If SSL Type is Let's Encrypt
				if [[ $ssltype == "LEHTTP" || $ssltype == "LEDNS" ]]; then
					# Copy LetsEncrypt config into local Standby Server
					cp -r /tmp/letsencrypt /etc
					
					# Copy VALT Scripts into local Standby Server
					cp -r /tmp/bin /usr/local/valt
					
					# Add the new Let's Encrypt renewal cronjob
					(crontab -l ; echo "0 3 * * * /usr/local/valt/bin/renewletsencrypt") | crontab -
					
				# If SSL Type is not Let's Encrypt, it it CA-Signed
				else
					mkdir /etc/apache2/ssl
					scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/etc/apache2/ssl/* /etc/apache2/ssl
				fi
				
				# Copy Apache SSL files from Primary Server to Standby Server
			#	scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/etc/apache2/sites-enabled/v3.conf /etc/apache2/sites-enabled/
				scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/etc/apache2/sites-enabled/valt.conf /etc/apache2/sites-enabled/
				
				# Copy Wowza SSL files from Primary Server to Standby Server
				scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/WowzaStreamingEngine/conf/valtwowza.jks /usr/local/WowzaStreamingEngine/conf/
				scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/WowzaStreamingEngine/java/lib/security/cacerts /usr/local/WowzaStreamingEngine/java/lib/security/
				scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/WowzaStreamingEngine/conf/VHost.xml /usr/local/WowzaStreamingEngine/conf/
				
				# Copy NodeJS SSL files from Primary Server to Standby Server
			#	scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/var/www/v3/nodejs/server.js /var/www/v3/nodejs/
				scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/var/www/valt/nodejs/build/server.js /var/www/valt/nodejs/build/
				
				# Set SSL firewall rules
				if ! iptables -S | grep ACCEPT | grep INPUT | grep 443; then
					iptables -A INPUT -p tcp -m tcp --dport 443 -j ACCEPT
				fi
				if ! iptables -S | grep ACCEPT | grep INPUT | grep 444; then
					iptables -A INPUT -p tcp -m tcp --dport 444 -j ACCEPT
				fi
				
				# Enable SSL Module and Headers Module for Apache
				a2enmod ssl
				a2enmod headers
			fi

			iptables -A OUTPUT -p tcp -m tcp --dport 22 -j ACCEPT
			iptables -A OUTPUT -p tcp -m tcp --dport 3306 -j ACCEPT
			netfilter-persistent save
			cp /usr/local/valt/conf/sql.cnf /usr/local/valt/conf/standby.cnf
			
			echo -e "[client]" > /usr/local/valt/conf/primary.cnf
			echo -e "host=$PRIMARY_SERVER_IP_ADDRESS" >> /usr/local/valt/conf/primary.cnf
			echo -e "user=$primaryuser" >> /usr/local/valt/conf/primary.cnf
			echo -e "password=$primarypass" >> /usr/local/valt/conf/primary.cnf
			
		#	mysqldump --defaults-extra-file=/usr/local/valt/conf/standby.cnf --opt v3 > /usr/local/valt/backup/v3_orig.sql
			mysqldump --defaults-extra-file=/usr/local/valt/conf/standby.cnf --opt $DATABASE_NAME > /usr/local/valt/backup/valt_orig.sql
		#	mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "DROP DATABASE v3;"
			mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "DROP DATABASE $DATABASE_NAME;"
		#	mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "CREATE DATABASE v3;"
			mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "CREATE DATABASE $DATABASE_NAME;"
		#	scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/valt/backup/v3_sync.sql /usr/local/valt/backup/v3_sync.sql
			scp ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/valt/backup/valt_sync.sql /usr/local/valt/backup/valt_sync.sql
		#	mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf v3 < /usr/local/valt/backup/v3_sync.sql
			mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf valt < /usr/local/valt/backup/valt_sync.sql
			echo -e "server-id\t\t= 2" >> /etc/mysql/mysql.conf.d/mysqld.cnf
			echo -e "relay-log\t\t= /var/log/mysql/mysql-relay-bin.log" >> /etc/mysql/mysql.conf.d/mysqld.cnf
			echo -e "log_bin\t\t\t= /var/log/mysql/mysql-bin.log" >> /etc/mysql/mysql.conf.d/mysqld.cnf
		#	echo -e "binlog_do_db\t\t= v3" >> /etc/mysql/mysql.conf.d/mysqld.cnf
			echo -e "binlog_do_db\t\t= valt" >> /etc/mysql/mysql.conf.d/mysqld.cnf
			service mysql restart
			mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "CHANGE MASTER TO MASTER_HOST='$PRIMARY_SERVER_IP_ADDRESS', MASTER_USER='$SLAVE_USER', MASTER_PASSWORD='$SLAVE_PASSWORD', MASTER_LOG_FILE='$masterlogfile', MASTER_LOG_POS=$masterlogposition;"
			mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "START SLAVE;"
			cp failure.sh /usr/local/valt/bin/
			cp restore.sh /usr/local/valt/bin/
			chmod +x /usr/local/valt/bin/failure.sh
			chmod +x /usr/local/valt/bin/restore.sh
			
			# Failover Standby Server configured successfully
			isStandbyServer=true
			
			UpdateWarmStandbyVariables
			
			dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Configure Standby Server" --msgbox "Standby Server configured successfully. Warm Standby Configuration completed." 6 50
			
		# Primary Server has not been successfully configured yet	
		else
			dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Configure Standby Server" --msgbox "You must run VALTPrimaryServerConfig5.5.sh on the Primary Standy Server first!" 6 50
		fi
		
	fi
}

# Load Environment Variables
LoadEnvironmentVariables

# Configure Standby Server
ConfigureStandbyServer
