#!/bin/bash
# This script will migrate the database and all recordings back to the primary VALT server. 
# The DNS entry must be updated to point to the primary VALT server.

# Load VALT Environment Variables
function LoadEnvironmentVariables() {
	source /usr/local/valt/conf/valtpath
	source $valtpath/.env.local
}

# Update VALT Environment Variables
function UpdateEnvLocal () {
	name=$1
	value="${2//\//\\/}"
	if [[ ! -f $valtpath/.env.local ]]; then
		echo -e "$1=$2" > $valtpath/.env.local
	else
		sed -i -e '/^#\?\(\s*'"${name}"'\s*=\s*\).*/{s//\1'"${value}"'/;:a;n;ba;q}' -e '$a'"${name}"'='"${value}" $valtpath/.env.local
	fi
}

# Update Environment Variables with Warm Standby variables
function UpdateWarmStandbyVariables() {
	UpdateEnvLocal "IS_PRIMARY_SERVER" "$isPrimaryServer"
	UpdateEnvLocal "IS_STANDBY_SERVER" "$isStandbyServer"
	UpdateEnvLocal "PRIMARY_SERVER_IP_ADDRESS" "$primaryserveripaddress"
	UpdateEnvLocal "STANDBY_SERVER_IP_ADDRESS" "$standbyserveripaddress"
	UpdateEnvLocal "SLAVE_USER" "$slaveuser"
	UpdateEnvLocal "SLAVE_PASSWORD" "$slavepassword"
}

#primaryip="10.111.3.73"
#slaveuser="ivs_user"
#slavepassword="admin51"

echo "WARNING: DO NOT RUN THIS SCRIPT MULTIPLE TIMES!!!"
echo "MAKE SURE THE DNS ENTRY HAS BEEN UPDATED PRIOR TO EXECUTING THIS SCRIPT"
echo "Press enter to continue."
read r
export NCURSES_NO_UTF8_ACS=1 
if [[ $EUID -ne 0 ]]; then
  	echo "You must run this script as root" 2>&1
  	exit 1
else
	# Get Configuration information from .env.local
	LoadEnvironmentVariables
	
	# Verify this server is a Standby Server
	if [[ $IS_STANDBY_SERVER == "true" ]]; then
		systemd-resolve --flush-caches
		service apache2 stop
		service WowzaStreamingEngine stop
		service valtcontrols stop
		systemctl disable apache2 
		systemctl disable valtcontrols
		systemctl disable WowzaStreamingEngine
		rsync --rsync-path="sudo rsync" -avz --progress /usr/local/WowzaStreamingEngine/content/valt_recordings ivsadmin@$PRIMARY_SERVER_IP_ADDRESS:/usr/local/WowzaStreamingEngine/content/
	#	mysqldump --defaults-extra-file=/usr/local/valt/conf/standby.cnf --opt v3 > /usr/local/valt/backup/v3.sql
		mysqldump --defaults-extra-file=/usr/local/valt/conf/standby.cnf --opt valt > /usr/local/valt/backup/valt.sql
	#	mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf -e "DROP DATABASE v3;"
		mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf -e "DROP DATABASE valt;"	
	#	mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf -e "CREATE DATABASE v3;"
		mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf -e "CREATE DATABASE valt;"	
	#	mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf v3 < /usr/local/valt/backup/v3.sql
		mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf valt < /usr/local/valt/backup/valt.sql
	#	mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf v3 -e "FLUSH TABLES WITH READ LOCK;"
		mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf valt -e "FLUSH TABLES WITH READ LOCK;"
	#	tmp1=$(mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf v3 -e "SHOW MASTER STATUS\G;" | grep File)
		tmp1=$(mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf valt -e "SHOW MASTER STATUS\G;" | grep File)	
		sqlfile=${tmp1#*:}
	#	tmp1=$(mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf v3 -e "SHOW MASTER STATUS\G;" | grep Position)
		tmp1=$(mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf valt -e "SHOW MASTER STATUS\G;" | grep Position)
		position=${tmp1#*:}
		sqlfile=$(echo $sqlfile | xargs)
	#	mysqldump --defaults-extra-file=/usr/local/valt/conf/primary.cnf --opt v3 > /usr/local/valt/backup/v3_sync.sql
		mysqldump --defaults-extra-file=/usr/local/valt/conf/primary.cnf --opt valt > /usr/local/valt/backup/valt_sync.sql
	#	mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf v3 -e "UNLOCK TABLES;"
		mysql --defaults-extra-file=/usr/local/valt/conf/primary.cnf valt -e "UNLOCK TABLES;"
	#	mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "DROP DATABASE v3;"
		mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "DROP DATABASE valt;"
	#	mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "CREATE DATABASE v3;"
		mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "CREATE DATABASE valt;"
	#	mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf v3 < /usr/local/valt/backup/v3_sync.sql
		mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf valt < /usr/local/valt/backup/valt_sync.sql
		mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "CHANGE MASTER TO MASTER_HOST="\'"$PRIMARY_SERVER_IP_ADDRESS"\'", MASTER_USER="\'"$SLAVE_USER"\'", MASTER_PASSWORD="\'"$SLAVE_PASSWORD"\'", MASTER_LOG_FILE="\'"$sqlfile"\'", MASTER_LOG_POS = $position;"
		mysql --defaults-extra-file=/usr/local/valt/conf/standby.cnf -e "START SLAVE;"
		
		dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Restore Standby Server" --msgbox "Database and Recordings have been restored to the Primary Server located at $PRIMARY_SERVER_IP_ADDRESS." 8 50
	
	# This server is not a Standby Server
	else
		dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Restore Standby Server" --msgbox "This restoral script can only be ran on a STANDBY SERVER! This server is NOT configured as a standby server!" 8 50
	fi
fi
